# Copyright 2001 Benjamin Trott. This code cannot be redistributed without
# permission from www.movabletype.org.
#
# $Id: Blog.pm,v 1.18 2001/12/11 20:26:59 btrott Exp $

package MT::Blog;
use strict;

use MT::Object;
@MT::Blog::ISA = qw( MT::Object );
__PACKAGE__->install_properties({
    columns => [
        'id', 'name', 'description',
        'archive_path', 'archive_url', 'archive_type',
        'archive_type_preferred',
        'site_path', 'site_url',
        'days_on_index', 'file_extension',
        'email_new_comments', 'allow_comment_html',
        'autolink_urls', 'sort_order_posts', 'sort_order_comments',
        'allow_comments_default', 'server_offset', 'convert_paras',
        'convert_paras_comments',
        'status_default', 'allow_anon_comments', 'words_in_excerpt',
        'ping_weblogs', 'mt_update_key',
        'archive_tmpl_monthly', 'archive_tmpl_weekly', 'archive_tmpl_daily',
        'archive_tmpl_individual', 'archive_tmpl_category',
    ],
    indexes => {
        name => 1,
    },
    datasource => 'blog',
    primary_key => 'id',
});

{
    my %blogs;
    sub load {
        my($class, $terms, $args) = @_;
        if (!wantarray) {
            return $blogs{$terms} if !ref($terms) && $blogs{$terms};
            my $blog = $class->SUPER::load($terms, $args);
            return $blog ? ($blogs{$blog->id} = $blog) : undef;
        } else {
            return $class->SUPER::load($terms, $args);
        }
    }
}

sub remove {
    my $blog = shift;
    my @classes = qw( MT::Permission MT::Entry MT::Template
                      MT::Category MT::Notification );
    my $blog_id = $blog->id;
    for my $class (@classes) {
        eval "use $class;";
        ## We need to loop twice over the objects: first gather, then
        ## remove, so as not to throw our gathering out of whack by removing
        ## while we gather. :)
        my $iter = $class->load_iter({ blog_id => $blog_id });
        my @ids;
        while (my $obj = $iter->()) {
            push @ids, $obj->id;
        }
        ## The iterator is finished, so we can safely remove.
        for my $id (@ids) {
            my $obj = $class->load($id);
            $obj->remove;
        }
    }
    $blog->SUPER::remove;
}

1;
