# Copyright 2001 Benjamin Trott. This code cannot be redistributed without
# permission from www.movabletype.org.
#
# $Id: Builder.pm,v 1.12 2001/12/04 23:25:01 btrott Exp $

package MT::Builder;
use strict;

use MT::ErrorHandler;
@MT::Builder::ISA = qw( MT::ErrorHandler );

sub new { bless { }, $_[0] }

sub compile {
    my $build = shift;
    my($ctx, $text) = @_;
    return [ ] unless $text;
    my $state = local $build->{__state} = { tokens => [ ] };
    $state->{text} = \$text;
    my $pos = 0;
    my $len = length $text;
    while ($text =~ m!(<\$?MT([^>\$]+)\$?>)!g) {
        my($whole_tag, $tag) = ($1, $2);
        ($tag, my($args)) = split /\s+/, $tag, 2;
        my $sec_start = pos $text;
        my $tag_start = $sec_start - length $whole_tag;
        _text_block($state, $pos, $tag_start) if $pos < $tag_start;
        $args ||= '';
        my %args;
        while ($args =~ /(\w+)\s*=\s*(["'])(.*?)\2/g) {
            $args{$1} = $3;
        }
        my($h, $is_container) = $ctx->handler_for($tag);
        my $rec = [ $tag, \%args ];
        if ($is_container) {
            if ($text =~ m!(</MT$tag>)!g) {
                my $sec_end = pos($text) - length $1;
                my $sec = substr $text, $sec_start, $sec_end - $sec_start;
                $sec =~ s!^\n!!;
                $rec->[2] = $build->compile($ctx, $sec) or return;
            } else {
                return $build->error("<MT$tag> with no </MT$tag>");
            }
        }
        push @{ $state->{tokens} }, $rec;
        $pos = pos $text;
    }
    _text_block($state, $pos, $len) if $pos < $len;
    $state->{tokens};
}

sub _text_block {
    my $text = substr ${ $_[0]->{text} }, $_[1], $_[2] - $_[1];
    push @{ $_[0]->{tokens} }, [ 'TEXT', $text ] if $text;
}

sub build {
    my $build = shift;
    my($ctx, $tokens, $cond) = @_;
    $cond ||= { };
    $ctx->stash('builder', $build);
    my $res = '';
    my $ph = $ctx->post_process_handler;
    for my $t (@$tokens) {
        if ($t->[0] eq 'TEXT') {
            $res .= $t->[1];
        } else {
            next if exists $cond->{ $t->[0] } && !$cond->{ $t->[0] };
            my($h) = $ctx->handler_for($t->[0]);
            if ($h) {
                $ctx->stash('tag', $t->[0]);
                $ctx->stash('tokens', $t->[2]);
                my $out = $h->($ctx, $t->[1], $cond);
                return $build->error($ctx->errstr) unless defined $out;
                if ($ph) {
                    $out = $ph->($ctx, $t->[1], $out);
                }
                $res .= $out;
            }
        }
    }
    $res;
}

1;
