# Copyright 2001 Benjamin Trott. This code cannot be redistributed without
# permission from www.movabletype.org.
#
# $Id: Entry.pm,v 1.29 2001/10/24 23:56:11 btrott Exp $

package MT::Entry;
use strict;

use MT::Author;
use MT::Category;
use MT::Comment;
use MT::Util qw( archive_file_for );

use MT::Object;
@MT::Entry::ISA = qw( MT::Object );
__PACKAGE__->install_properties({
    columns => [
        'id', 'blog_id', 'status', 'author_id', 'allow_comments',
        'category_id', 'title', 'excerpt', 'text', 'text_more'
    ],
    indexes => {
        category_id => 1,
        blog_id => 1,
        status => 1,
        author_id => 1,
        created_on => 1,
    },
    audit => 1,
    datasource => 'entry',
    primary_key => 'id',
});

use constant HOLD    => 1;
use constant RELEASE => 2;
use constant ARCHIVE => 3;

use Exporter;
*import = \&Exporter::import;
use vars qw( @EXPORT_OK );
@EXPORT_OK = qw( HOLD RELEASE ARCHIVE );

sub next {
    my $entry = shift;
    unless ($entry->{__next}) {
        $entry->{__next} = MT::Entry->load(
            { blog_id => $entry->blog_id,
              status => RELEASE },
            { limit => 1,
              'sort' => 'created_on',
              direction => 'ascend',
              start_val => $entry->created_on });
    }
    $entry->{__next};
}

sub previous {
    my $entry = shift;
    unless ($entry->{__previous}) {
        $entry->{__previous} = MT::Entry->load(
            { blog_id => $entry->blog_id,
              status => RELEASE },
            { limit => 1,
              'sort' => 'created_on',
              direction => 'descend',
              start_val => $entry->created_on });
    }
    $entry->{__previous};
}

sub author {
    my $entry = shift;
    unless ($entry->{__author}) {
        $entry->{__author} = MT::Author->load($entry->author_id);
    }
    $entry->{__author};
}

sub category {
    my $entry = shift;
    return unless $entry->category_id;
    unless ($entry->{__category}) {
        $entry->{__category} = MT::Category->load($entry->category_id);
    }
    $entry->{__category};
}

sub comments {
    my $entry = shift;
    unless ($entry->{__comments}) {
        $entry->{__comments} = [ MT::Comment->load({
            entry_id => $entry->id
        }) ];
    }
    $entry->{__comments};
}

sub comment_count {
    my $entry = shift;
    unless ($entry->{__comment_count}) {
        $entry->{__comment_count} = MT::Comment->count({
            entry_id => $entry->id
        });
    }
    $entry->{__comment_count};
}

sub archive_file {
    my $entry = shift;
    my($at) = @_;
    my $blog_id = $entry->blog_id;
    require MT::Blog;
    my $blog = MT::Blog->load($blog_id) or
        return $entry->error("Load of blog '$blog_id' failed: " .
            MT::Blog->errstr);
    unless ($at) {
        $at = $blog->archive_type_preferred || $blog->archive_type;
        return '' if !$at || $at eq 'None';
        my %at = map { $_ => 1 } split /,/, $at;
        for my $tat (qw( Individual Daily Weekly Monthly Category )) {
            $at = $tat if $at{$tat};
        }
    }
    archive_file_for($entry, $blog, $at);
}

sub remove {
    my $entry = shift;
    my $comments = $entry->comments;
    for my $comment (@$comments) {
        $comment->remove;
    }
    $entry->SUPER::remove;
}

1;
