# Copyright 2001 Benjamin Trott. This code cannot be redistributed without
# permission from www.movabletype.org.
#
# $Id: Mail.pm,v 1.4 2001/12/02 22:16:11 btrott Exp $

package MT::Mail;
use strict;

use MT;
use MT::ConfigMgr;
use MT::ErrorHandler;
@MT::Mail::ISA = qw( MT::ErrorHandler );

sub send {
    my $class = shift;
    my($hdrs, $body) = @_;
    $body .= <<SIG;

-- 
@{[ MT->version_slug ]}
SIG
    my $mgr = MT::ConfigMgr->instance;
    my $xfer = $mgr->MailTransfer;
    if ($xfer eq 'sendmail') {
        return $class->_send_mt_sendmail($hdrs, $body, $mgr);
    } elsif ($xfer eq 'smtp') {
        return $class->_send_mt_smtp($hdrs, $body, $mgr);
    } elsif ($xfer eq 'debug') {
        return $class->_send_mt_debug($hdrs, $body, $mgr);
    } else {
        return $class->error("Unknown MailTransfer method '$xfer'");
    }
}

sub _send_mt_debug {
    my $class = shift;
    my($hdrs, $body, $mgr) = @_;
    for my $key (keys %$hdrs) {
        my @arr = ref($hdrs->{$key}) eq 'ARRAY' ?
            @{ $hdrs->{$key} } : ($hdrs->{$key});
        print STDERR map "$key: $_\n", @arr;
    }
    print STDERR "\n";
    print STDERR $body;
    1;
}

sub _send_mt_smtp {
    my $class = shift;
    my($hdrs, $body, $mgr) = @_;
    eval { require Mail::Sendmail; };
    return $class->error("Sending mail via SMTP requires that your server " .
                         "have Mail::Sendmail installed: $@") if $@;
    $hdrs->{Message} = $body;
    $hdrs->{Smtp} = $mgr->SMTPServer;
    my $ret = Mail::Sendmail::sendmail(%$hdrs);
    $ret or return $class->error("Error sending mail: " .
                                 $Mail::Sendmail::error);
    1;
}

my @Sendmail = qw( /usr/lib/sendmail /usr/sbin/sendmail /usr/ucblib/sendmail );
sub _send_mt_sendmail {
    my $class = shift;
    my($hdrs, $body, $mgr) = @_;
    my $sm_loc;
    for my $loc ($mgr->SendMailPath, @Sendmail) {
        next unless $loc;
        $sm_loc = $loc, last if -x $loc && !-d $loc;
    }
    return $class->error("You do not have a valid path to sendmail on " .
                         "your machine. Perhaps you should try using SMTP?")
        unless $sm_loc;
    local $SIG{PIPE} = { };
    my $pid = open MAIL, '|-';
    local $SIG{ALRM} = sub { CORE::exit() };
    return unless defined $pid;
    if (!$pid) {
        exec $sm_loc, "-t" or
            return $class->error("Exec of sendmail failed: $!");
    }
    for my $key (keys %$hdrs) {
        my @arr = ref($hdrs->{$key}) eq 'ARRAY' ?
            @{ $hdrs->{$key} } : ($hdrs->{$key});
        print MAIL map "$key: $_\n", @arr;
    }
    print MAIL "\n";
    print MAIL $body;
    close MAIL;
    1;
}

1;
