# Copyright 2001 Benjamin Trott. This code cannot be redistributed without
# permission from www.movabletype.org.
#
# $Id: XMLRPC.pm,v 1.4 2001/11/02 08:24:51 btrott Exp $

package MT::XMLRPC;
use strict;

use MT;
use MT::ConfigMgr;
use MT::Util qw( encode_html );
use MT::ErrorHandler;
@MT::XMLRPC::ISA = qw( MT::ErrorHandler );

sub weblogs_ping {
    my $class = shift;
    my($blog) = @_;
    my $url = MT::ConfigMgr->instance->WeblogsPingURL
        or return $class->error("No WeblogsPingURL defined in mt.cfg");
    $class->_ping_update('weblogUpdates.ping', $blog, $url);
}

sub mt_ping {
    my $class = shift;
    my($blog) = @_;
    my $url = MT::ConfigMgr->instance->MTPingURL
        or return $class->error("No MTPingURL defined in mt.cfg");
    if (!ref($blog)) {
        require MT::Blog;
        $blog = MT::Blog->load($blog);
    }
    $class->_ping_update('mtUpdates.ping', $blog, $url, $blog->mt_update_key);
}

sub _ping_update {
    my $class = shift;
    my($method, $blog, $url, $mt_key) = @_;
    if (!ref($blog)) {
        require MT::Blog;
        $blog = MT::Blog->load($blog);
    }
    require LWP::UserAgent;
    my $ua = LWP::UserAgent->new;
    $ua->agent('MovableType/' . MT->VERSION);
    my $req = HTTP::Request->new('POST', $url);
    $req->header('Content-Type' => 'text/xml');
    my $blog_name = encode_html($blog->name);
    my $blog_url = encode_html($blog->site_url);
    my $meth = $mt_key ? "mtUpdates.ping" : "weblogUpdates.ping";
    my $text = <<XML;
<?xml version="1.0"?>
<methodCall>
    <methodName>$method</methodName>
    <params>
    <param><value>$blog_name</value></param>
    <param><value>$blog_url</value></param>
XML
    if ($mt_key) {
        $text .= "    <param><value>$mt_key</value></param>\n";
    }
    $text .= <<XML;
    </params>
</methodCall>
XML
    $req->content($text);
    my $res = $ua->request($req);
    if (substr($res->code, 0, 1) ne '2') {
        return $class->error( "HTTP error: " . $res->status_line );
    }
    my $content = $res->content;
    my($error, $msg) = $content =~
        m!flerror.*?<boolean>(\d+).*message.*?<value>([^>]+)</value>!s;
    if ($error) {
        return $class->error( "Ping error: " . $msg );
    }
    $msg;
}

1;
