#!/usr/bin/perl -w

# Copyright 2001 Benjamin Trott. This code cannot be redistributed without
# permission from www.movabletype.org.
#
# $Id: mt-check.cgi,v 1.9 2001/12/07 07:52:31 btrott Exp $
use strict;

local $|=1;

my($MT_DIR);
BEGIN {
    if ($0 =~ m!(.*[/\\])!) {
        $MT_DIR = $1;
    } else {
        $MT_DIR = './';
    }
    unshift @INC, $MT_DIR . 'lib';
    unshift @INC, $MT_DIR . 'extlib';
}

print "Content-Type: text/html\n\n";
print "<pre>\n";

my $is_good = 1;

my @REQ = (
    [ 'HTML::Template', 2, 1, 'HTML::Template is required for all Movable Type application functionality.' ],

    [ 'Image::Size', 0, 1, 'Image::Size is required for file uploads (to determine the size of uploaded images in many different formats.' ],

    [ 'File::Spec', 0, 1, 'File::Spec is required for path manipulation across operating systems.' ],

    [ 'DB_File', 0, 1, 'DB_File is required for backend database functionality.' ],

    [ 'CGI::Cookie', 0, 1, 'CGI::Cookie is required for cookie authentication.' ],
);

my @OPT = (
    [ 'LWP::UserAgent', 0, 0, 'LWP::UserAgent is optional; it is needed if you wish to use either the weblogs.com ping or the MT Recently Updated ping.' ],

    [ 'SOAP::Lite', 0, 0, 'SOAP::Lite is optional; it is needed if you wish to use the MT XML-RPC server implementation.' ],

    [ 'File::Temp', 0, 0, 'File::Temp is optional; it is needed if you would like to be able to overwrite existing files when you upload.' ],
);

print <<HTML;
Movable Type [mt-check.cgi]

HTML

use Text::Wrap;
$Text::Wrap::columns = 72;

for my $list (\@REQ, \@OPT) {
    my $req = 1 if $list == \@REQ;
    printf "CHECKING FOR %s MODULES:\n\n", $req ? "REQUIRED" : "OPTIONAL";
    if (!$req) {
        print <<MSG;
The following modules are optional; if your server does not have these
modules installed, you only need to install them if you require the
functionality that the module provides.

MSG
    }
    for my $ref (@$list) {
        my($mod, $ver, $desc) = @$ref;
        print "    $mod" .
            ($ver ? " (version &gt;= $ver)" : "") . "...\n";
        eval("use $mod" . ($ver ? " $ver;" : ";"));
        if ($@) {
            $is_good = 0 if $req;
            my $msg = $ver ?
                      "Either your server does not have $mod installed, or " .
                      "the version that is installed is too old. " :
                      "Your server does not have $mod installed. ";
            $msg   .= $desc .
                      " Please consult the installation instructions for " .
                      "help in installing $mod.";
            print wrap("        ", "        ", $msg), "\n\n";
        } else {
            print "        Your server has $mod installed.\n\n";
        }
    }
    print "\n";
}

if ($is_good) {
    print <<HTML;
Your server has all of the required modules installed; you do not need to
perform any additional module installations. Continue with the installation
instructions in the section "Setting up your Blog Directories".
HTML
}

print "</pre>\n";
