#!/usr/bin/perl -w

# Copyright 2001 Benjamin Trott. This code cannot be redistributed without
# permission from www.movabletype.org.
#
# $Id: mt-comments.cgi,v 1.26 2001/12/13 01:51:26 btrott Exp $
use strict;

my($MT_DIR);
BEGIN {
    if ($0 =~ m!(.*[/\\])!) {
        $MT_DIR = $1;
    } else {
        $MT_DIR = './';
    }
    unshift @INC, $MT_DIR . 'lib';
    unshift @INC, $MT_DIR . 'extlib';
}

eval {
    require CGI;
    require MT::Comment;
    require MT;

    my $mt = MT->new( Config => $MT_DIR . 'mt.cfg' );
    my $q = CGI->new;

    if ($q->param('post')) {
        if (my $state = $q->param('comment_state')) {
            require MT::Serialize;
            my $ser = MT::Serialize->new($mt->{cfg}->Serializer);
            $state = $ser->unserialize(pack 'H*', $state);
            $state = $$state;
            for my $f (keys %$state) {
                $q->param($f, $state->{$f});
            }
        }
        require MT::Entry;
        my $entry = MT::Entry->load(scalar $q->param('entry_id')) or
            die "Invalid entry ID.\n";
        require MT::Blog;
        my $blog = MT::Blog->load($entry->blog_id);
        if (!$blog->allow_anon_comments &&
            (!$q->param('author') || !$q->param('email'))) {
            die "Author name and email address are required.\n";
        }
        my $comment = MT::Comment->new;
        $comment->blog_id($entry->blog_id);
        $comment->entry_id($q->param('entry_id'));
        $comment->author($q->param('author'));
        my $email = $q->param('email') || '';
        if ($email) {
            if ($email =~ /[ |\t|\r|\n]*\"?([^\"]+\"?@[^ <>\t]+\.[^ <>\t][^ <>\t]+)[ |\t|\r|\n]*/) {
                $email = $1;
            } else {
                die "Invalid email address '$email'\n";
            }
        }
        $comment->email($email);
        my $url = $q->param('url') || '';
        if ($url) {
            if ($url =~ /^(([^:\/?#]+):)?(\/\/([^\/?#]*))?([^?#]*)(\?([^#]*))?(#(.*))?/) {

                $url = (($2) && ($2 eq "http"))  ? "$1$3$5$6" : "http://$5$6";
            } else {
                die "Invalid URL '$url'\n";
            }
        }
        $comment->url($url);
        $comment->text($q->param('text'));
        $comment->save;
        $mt->rebuild_indexes( Blog => $blog ) or
            die "Index rebuild failed: ", $mt->errstr;
        $mt->rebuild_entry( Entry => $entry ) or
            die "Rebuild failed: ", $mt->errstr;
        my $link_url;
        if (!$q->param('static')) {
            my $url = $mt->{cfg}->CGIPath;
            $url .= '/' unless substr($url, -1, 1) eq '/';
            $url .= 'mt-comments.cgi?entry_id=' . $q->param('entry_id');
            print $q->redirect($url);
            $link_url = $url;
        } else {
            my $url = $blog->archive_url;
            $url .= '/' unless $url =~ m!/$!;
            $url .= $entry->archive_file;
            print $q->redirect($url);
            $link_url = $url;
        }
        if ($blog->email_new_comments) {
            require MT::Mail;
            my $author = $entry->author;
            if ($author->email) {
                my %head = ( To => $author->email,
                             From => $comment->email || $author->email,
                             Subject =>
                                 '[' . $blog->name . '] New Comment Posted' );
                MT::Mail->send(\%head, <<BODY);
A new comment has been posted on your blog @{[ $blog->name ]}, on
entry #@{[ $entry->id ]} (@{[ $entry->title ]}).

$link_url

Name: @{[ $comment->author ]}
Email Address: @{[ $comment->email ]}
URL: @{[ $comment->url ]}

Comments:

@{[ $comment->text ]}
BODY
            }
        }
    }
    elsif ($q->param('preview')) {
        require MT::Template;
        require MT::Template::Context;
        require MT::Entry;
        require MT::Util;
        require MT::Comment;
        my $entry_id = $q->param('entry_id');
        my $entry = MT::Entry->load($entry_id);
        my $ctx = MT::Template::Context->new;
        my $comment = MT::Comment->new;
        $comment->blog_id($entry->blog_id);
        for my $f (qw( entry_id author email url text )) {
            $comment->$f(scalar $q->param($f));
        }
        ## Set timestamp as we would usually do in ObjectDriver.
        my @ts = MT::Util::offset_time_list(time, $entry->blog_id);
        my $ts = sprintf "%04d%02d%02d%02d%02d%02d",
            $ts[5]+1900, $ts[4]+1, @ts[3,2,1,0];
        $comment->created_on($ts);
        $ctx->stash('comment_preview', $comment);
        ## Serialize comment state, then hex-encode it.
        require MT::Serialize;
        my $ser = MT::Serialize->new($mt->{cfg}->Serializer);
        my $state = $comment->column_values;
        $state->{static} = $q->param('static');
        $ctx->stash('comment_state', unpack 'H*', $ser->serialize(\$state));
        $ctx->stash('entry', $entry);
        $ctx->{current_timestamp} = $ts;
        my $tmpl = MT::Template->load({ type => 'comment_preview',
                                        blog_id => $entry->blog_id });
        die "You must define a Comments Preview template in order to " .
            "display comment previews." unless $tmpl;
        my $html = $tmpl->build($ctx);
        $html = $tmpl->errstr unless defined $html;
        print $q->header;
        print $html;
    }
    else {
        require MT::Template;
        require MT::Template::Context;
        require MT::Entry;
        my $entry_id = $q->param('entry_id');
        my $entry = MT::Entry->load($entry_id);
        my $ctx = MT::Template::Context->new;
        $ctx->stash('entry', $entry);
        my $tmpl = MT::Template->load({ type => 'comments',
                                        blog_id => $entry->blog_id });
        die "You must define a Comments Listing template in order to " .
            "display dynamic comments." unless $tmpl;
        my $html = $tmpl->build($ctx);
        $html = $tmpl->errstr unless defined $html;
        print $q->header;
        print $html;
    }
};
if ($@) {
    print "Content-Type: text/html\n\n";
    print "An error occurred: $@";
}
