#!/usr/bin/perl -w

# Copyright 2001 Benjamin Trott. This code cannot be redistributed without
# permission from www.movabletype.org.
#
# $Id: mt-export.cgi,v 1.3 2001/12/07 22:24:19 btrott Exp $
use strict;

local $|=1;
print "Content-Type: text/plain\n\n";

my($MT_DIR);
BEGIN {
    if ($0 =~ m!(.*[/\\])!) {
        $MT_DIR = $1;
    } else {
        $MT_DIR = './';
    }
    unshift @INC, $MT_DIR . 'lib';
    unshift @INC, $MT_DIR . 'extlib';
}

eval {
    require MT::App::CMS;
    require MT::Entry;
    require MT::Permission;

    my $mt = MT::App::CMS->new( Config => $MT_DIR . 'mt.cfg' );

    my($author, $first_time) = $mt->login;
    unless ($author) {
        die "You are not logged in. Please log in through the main\n" .
            "Movable Type system, then export your files through there.\n";
    }

    my $q = $mt->{query};
    my $SEP = ('-' x 8);
    my $SUB_SEP = ('-' x 5);

    require MT::Blog;
    my $blog_id = $q->param('blog_id') or die "No blog ID";
    my $blog = MT::Blog->load($blog_id)
        or die "Blog load failed: ", MT::Blog->errstr;
    $blog->convert_paras(0);
    $blog->convert_paras_comments(0);

    my $author_id = $author->id;
    my $perms = MT::Permission->load({ author_id => $author_id,
                                       blog_id => $blog_id });
    die "You do not have export permissions"
        unless $perms && $perms->can_edit_config;

    ## Create template for exporting a single entry
    require MT::Template;
    require MT::Template::Context;
    my $tmpl = MT::Template->new;
    $tmpl->blog_id($blog_id);
    $tmpl->name('Export Template');
    $tmpl->text(<<'TEXT');
AUTHOR: <$MTEntryAuthor$>
TITLE: <$MTEntryTitle$>
CATEGORY: <$MTEntryCategory$>
DATE: <$MTEntryDate format="%m/%d/%Y %I:%M:%S %p"$>
-----
BODY:
<$MTEntryBody$>
-----
EXTENDED BODY:
<$MTEntryMore$>
-----
EXCERPT:
<$MTEntryExcerpt no_generate="1"$>
-----
<MTComments>
COMMENT:
AUTHOR: <$MTCommentAuthor$>
EMAIL: <$MTCommentEmail$>
URL: <$MTCommentURL$>
DATE: <$MTCommentDate format="%m/%d/%Y %I:%M:%S %p"$>
<$MTCommentBody$>
-----
</MTComments>
--------
TEXT

    my $iter = MT::Entry->load_iter({ blog_id => $blog_id });

    while (my $entry = $iter->()) {
        my $ctx = MT::Template::Context->new;
        $ctx->stash('entry', $entry);
        $ctx->{current_timestamp} = $entry->created_on;
        my $res = $tmpl->build($ctx)
            or die $tmpl->errstr;
        print $res;
    }
};
if ($@) {
    print "An error occurred while importing your data:\n$@";
}
