#!/usr/bin/perl -w

# Copyright 2001 Benjamin Trott. This code cannot be redistributed without
# permission from www.movabletype.org.
#
# $Id: mt-import.cgi,v 1.20 2001/12/07 22:24:19 btrott Exp $
use strict;

local $|=1;
print "Content-Type: text/html\n\n";

my($MT_DIR);
BEGIN {
    if ($0 =~ m!(.*[/\\])!) {
        $MT_DIR = $1;
    } else {
        $MT_DIR = './';
    }
    unshift @INC, $MT_DIR . 'lib';
    unshift @INC, $MT_DIR . 'extlib';
}

eval {
    print "<pre>\n";

    require MT::App::CMS;
    require MT::Entry;
    require MT::Author;
    require MT::Permission;
    require MT::Util;
    require File::Spec;
    require CGI;

    my $mt = MT::App::CMS->new( Config => $MT_DIR . 'mt.cfg' );

    my($author, $first_time) = $mt->login;
    unless ($author) {
        die "You are not logged in. Please log in through the main\n" .
            "Movable Type system, then import your files through there.\n";
    }

    my $q = $mt->{query};
    my $SEP = ('-' x 8);
    my $SUB_SEP = ('-' x 5);

    my(%authors, %categories);
    require MT::Blog;
    my $blog_id = $q->param('blog_id') or die "No blog ID";
    my $blog = MT::Blog->load($blog_id)
        or die "Blog load failed: ", MT::Blog->errstr;

    ## Determine the author as whom we will import the entries.
    my $author_id = $author->id;
    my $perms = MT::Permission->load({ author_id => $author_id,
                                       blog_id => $blog_id });
    die "You do not have import permissions"
        unless $perms && $perms->can_edit_config;
    print "Importing entries into blog '", $blog->name, "'\n";
    my($pass);
    my $import_as_me = $q->param('import_as_me');
    if ($import_as_me) {
        print "Importing entries as author '", $author->name, "'\n";
    } else {
        $pass = $q->param('password')
            or die "You need to provide a password if you are going to\n" .
                   "create new authors for each author listed in your blog.\n";
        print "Creating new authors for each author found in the blog\n";
    }
    print "\n";

    ## Default category for imported entries.
    my $def_cat_id = $q->param('default_cat_id');

    my $t_start = $q->param('title_start');
    my $t_end = $q->param('title_end');
    my $allow_comments = $blog->allow_comments_default ? 1 : 0;

    my $dir = $MT_DIR . 'import';
    opendir DH, $dir or die "Can't open directory '$dir': $!";
    for my $f (readdir DH) {
        next if $f =~ /^\./;
        print "Importing entries from file '$f'\n";
        my $file = File::Spec->catfile($dir, $f);
        open FH, $file or die "Can't open file '$file': $!";
        local $/ = $SEP;
        ENTRY_BLOCK:
        while (<FH>) {
            my($meta, @pieces) = split /$SUB_SEP/;
            next unless $meta && @pieces;

            ## Create entry object and assign some defaults.
            my $entry = MT::Entry->new;
            $entry->blog_id($blog_id);
            $entry->status(MT::Entry::RELEASE());
            $entry->allow_comments($allow_comments);
            $entry->category_id($def_cat_id) if $def_cat_id;
            $entry->author_id($author_id) if $import_as_me;

            ## Some users may want to import just their GM comments, having
            ## already imported their GM entries. We try to match up the
            ## entries using the created on timestamp, and the import file
            ## tells us not to import an entry with the meta-tag "NO ENTRY".
            my $no_save = 0;

            ## Handle all meta-data: author, category, title, date.
            my $i = -1;
            my @lines = split /\r?\n/, $meta;
            META:
            for my $line (@lines) {
                $i++;
                next unless $line;
                $line =~ s!^\s*!!;
                $line =~ s!\s*$!!;
                my($key, $val) = split /\s*:\s*/, $line, 2;
                if ($key eq 'AUTHOR' && !$import_as_me) {
                    my $author;
                    unless ($author = $authors{$val}) {
                        $author = MT::Author->load({ name => $val });
                    }
                    unless ($author) {
                        $author = MT::Author->new;
                        $author->name($val);
                        $author->set_password($pass);
                        print "Creating new author ('$val')...";
                        if ($author->save) {
                            print "ok\n";
                        } else {
                            print "failed\n";
                            die "Author save failed: ", $author->errstr;
                        }
                        $authors{$val} = $author;
                        print "Assigning permissions for new author...";
                        my $perms = MT::Permission->new;
                        $perms->blog_id($blog_id);
                        $perms->author_id($author->id);
                        $perms->can_post(1);
                        if ($perms->save) {
                            print "ok\n";
                        } else {
                            print "failed\n";
                            die "Permission save failed: ", $perms->errstr;
                        }
                    }
                    $entry->author_id($author->id);
                } elsif ($key eq 'CATEGORY') {
                    if ($val) {
                        my $cat;
                        unless ($cat = $categories{$val}) {
                            $cat = MT::Category->load({ label => $val });
                        }
                        unless ($cat) {
                            $cat = MT::Category->new;
                            $cat->blog_id($blog_id);
                            $cat->label($val);
                            $cat->author_id($entry->author_id);
                            print "Creating new category ('$val')...";
                            if ($cat->save) {
                                print "ok\n";
                            } else {
                                print "failed\n";
                                die "Category save failed: ", $cat->errstr;
                            }
                            $categories{$val} = $cat;
                        }
                        $entry->category_id($cat->id);
                    }
                } elsif ($key eq 'TITLE') {
                    $entry->title($val);
                } elsif ($key eq 'DATE') {
                    $entry->created_on(_convert_date($val));
                } elsif ($key eq 'NO ENTRY') {
                    $no_save++;
                } elsif ($key eq 'START BODY') {
                    ## Special case for backwards-compatibility with old
                    ## export files: if we see START BODY: on a line, we
                    ## gather up the rest of the lines in meta and package
                    ## them for handling below in the non-meta area.
                    @pieces = ("BODY:\n" . join "\n", @lines[$i+1..$#lines]);
                    last META;
                }
            }

            ## If we're not saving this entry (but rather just using it to
            ## import comments, for example), we need to load the relevant
            ## entry using the timestamp.
            if ($no_save) {
                my $ts = $entry->created_on;
                $entry = MT::Entry->load({ created_on => $ts });
                if (!$entry) {
                    print "Can't find existing entry with timestamp '$ts'... ",
                          "skipping comments, and moving on to next entry.\n";
                    next ENTRY_BLOCK;
                } else {
                    printf "Importing comments into existing entry %d ('%s')\n",
                        $entry->id, $entry->title;
                }
            }

            ## Deal with non-meta pieces: entry body, extended entry body,
            ## comments. We need to hold the list of comments until after
            ## we have saved the entry, then assign the new entry ID of
            ## the entry to each comment.
            my(@comments);
            for my $piece (@pieces) {
                $piece =~ s!^\s*!!;
                $piece =~ s!\s*$!!;
                if ($piece =~ s/^BODY:\r?\n//) {
                    $entry->text($piece);
                }
                elsif ($piece =~ s/^EXTENDED BODY:\r?\n//) {
                    $entry->text_more($piece);
                }
                elsif ($piece =~ s/^EXCERPT:\r?\n//) {
                    $entry->excerpt($piece) if $piece =~ /\S/;
                }
                elsif ($piece =~ s/^COMMENT:\r?\n//) {
                    ## Comments are: AUTHOR, EMAIL, URL, DATE (in any order),
                    ## then body
                    my $comment = MT::Comment->new;
                    $comment->blog_id($blog_id);
                    my @lines = split /\r?\n/, $piece;
                    my($i, $body_idx) = (0) x 2;
                    COMMENT:
                    for my $line (@lines) {
                        $line =~ s!^\s*!!;
                        my($key, $val) = split /\s*:\s*/, $line, 2;
                        if ($key eq 'AUTHOR') {
                            $comment->author($val);
                        } elsif ($key eq 'EMAIL') {
                            $comment->email($val);
                        } elsif ($key eq 'URL') {
                            $comment->url($val);
                        } elsif ($key eq 'DATE') {
                            $comment->created_on(_convert_date($val));
                        } else {
                            ## Now we have reached the body of the comment;
                            ## everything from here until the end of the
                            ## array is body.
                            $body_idx = $i;
                            last COMMENT;
                        }
                        $i++;
                    }
                    $comment->text( join "\n", @lines[$body_idx..$#lines] );
                    push @comments, $comment;
                }
            }

            ## Assign a title if one is not already assigned.
            unless ($entry->title) {
                my $body = $entry->text;
                if ($t_start && $t_end && $body =~
                    s!\Q$t_start\E(.*?)\Q$t_end\E\s*!!) {
                    $entry->title($1);
                } else {
                    $entry->title( MT::Util::first_n_words($body, 5) );
                }
            }

            ## Save entry.
            unless ($no_save) {
                print "Creating new entry ('", $entry->title, "')... ";
                if ($entry->save) {
                    print "ok (ID ", $entry->id, ")\n";
                } else {
                    print "failed\n";
                    die "Entry save failed: ", $entry->errstr;
                }
            }

            ## Save comments.
            for my $comment (@comments) {
                $comment->entry_id($entry->id);
                print "Creating new comment ('", $comment->author, "')... ";
                if ($comment->save) {
                    print "ok (ID ", $comment->id, ")\n";
                } else {
                    print "failed\n";
                    die "Comment save failed: ", $comment->errstr;
                }
            }
        }
    }
    closedir DH;
};
if ($@) {
    print "\nAn error occurred while importing your data:\n$@";
} else {
    print "\nAll data imported successfully! Make sure that you remove\n" .
          "the files that you imported from the 'import' folder, so that\n" .
          "if/when you run mt-import.cgi again, those files will not be\n" .
          "re-imported.\n";
}

print "</pre>\n";

sub _convert_date {
    my($date) = @_;
    my($mo, $d, $y, $h, $m, $s, $ampm) = $date =~
        m!^(\d{1,2})/(\d{1,2})/(\d{2,4}) (\d{1,2}):(\d{1,2}):(\d{1,2}) (\w{2})$!
            or die "Invalid date format '$date'; must be MM/DD/YYYY " .
                   "HH:MM:SS AM|PM\n";
    if ($ampm eq 'PM' && $h < 12) {
        $h += 12;
    } elsif ($ampm eq 'AM' && $h == 12) {
        $h = 0;
    }
    if (length($y) == 2) {
        $y += 1900;
    }
    sprintf "%04d%02d%02d%02d%02d%02d", $y, $mo, $d, $h, $m, $s;
}
